import { NestFactory } from '@nestjs/core';
import { MicroserviceOptions, Transport } from '@nestjs/microservices';
import { join } from 'path';
import { AppModule } from './app.module';

async function bootstrap() {
  const app = await NestFactory.create(AppModule);
  app.connectMicroservice<MicroserviceOptions>({
    transport: Transport.TCP,
  });
  app.connectMicroservice<MicroserviceOptions>({
    transport: Transport.GRPC,
    options: {
      package: ['ad', 'health'],
      protoPath: [
        join(__dirname, 'modules/ad/interface/grpc-controllers/ad.proto'),
        join(__dirname, 'health.proto'),
      ],
      url: `${process.env.SERVICE_URL}:${process.env.SERVICE_PORT}`,
      loader: { keepCase: true, enums: String },
    },
  });

  await app.startAllMicroservices();
  await app.listen(process.env.HEALTH_SERVICE_PORT);
}
bootstrap();
