import { Injectable } from '@nestjs/common';
import { DateTime, TimeZone } from 'timezonecomplete';
import { TimeConverterPort } from '../core/application/ports/time-converter.port';

@Injectable()
export class TimeConverter implements TimeConverterPort {
  private readonly UNIX_EPOCH = '1970-01-01';

  localStringTimeToUtcStringTime = (time: string, timezone: string): string => {
    try {
      if (!time || !timezone) throw new Error();
      return new DateTime(`${this.UNIX_EPOCH}T${time}`, TimeZone.zone(timezone))
        .convert(TimeZone.zone('UTC'))
        .format('HH:mm');
    } catch (e) {
      return undefined;
    }
  };

  utcStringTimeToLocalStringTime = (time: string, timezone: string): string => {
    try {
      if (!time || !timezone) throw new Error();
      return new DateTime(`${this.UNIX_EPOCH}T${time}`, TimeZone.zone('UTC'))
        .convert(TimeZone.zone(timezone))
        .format('HH:mm');
    } catch (e) {
      return undefined;
    }
  };

  localStringDateTimeToUtcDate = (
    date: string,
    time: string,
    timezone: string,
    dst = true,
  ): Date => {
    try {
      if (!date || !time || !timezone) throw new Error();
      return new Date(
        new DateTime(
          `${date}T${time}`,
          TimeZone.zone(timezone, dst),
        ).toIsoString(),
      );
    } catch (e) {
      return undefined;
    }
  };

  utcStringDateTimeToLocalIsoString = (
    date: string,
    time: string,
    timezone: string,
    dst?: boolean,
  ): string => {
    try {
      if (!date || !time || !timezone) throw new Error();
      return new DateTime(`${date}T${time}`, TimeZone.zone('UTC'))
        .convert(TimeZone.zone(timezone, dst))
        .toIsoString();
    } catch (e) {
      return undefined;
    }
  };

  utcUnixEpochDayFromTime = (time: string, timezone: string): number => {
    try {
      if (!time || !timezone) throw new Error();
      return new Date(
        new DateTime(
          `${this.UNIX_EPOCH}T${time}`,
          TimeZone.zone(timezone, false),
        )
          .convert(TimeZone.zone('UTC'))
          .toIsoString()
          .split('T')[0],
      ).getDay();
    } catch (e) {
      return undefined;
    }
  };

  localUnixEpochDayFromTime = (time: string, timezone: string): number => {
    try {
      if (!time || !timezone) throw new Error();
      return new Date(
        new DateTime(`${this.UNIX_EPOCH}T${time}`, TimeZone.zone('UTC'))
          .convert(TimeZone.zone(timezone))
          .toIsoString()
          .split('T')[0],
      ).getDay();
    } catch (e) {
      return undefined;
    }
  };
}
