import { hasValidPositionIndexes } from '@modules/ad/interface/grpc-controllers/dtos/validators/has-valid-position-indexes.validator';
import { WaypointDto } from '@modules/ad/interface/grpc-controllers/dtos/waypoint.dto';

describe('addresses position validator', () => {
  const mockAddress1: WaypointDto = {
    lat: 48.689445,
    lon: 6.17651,
    houseNumber: '5',
    street: 'Avenue Foch',
    locality: 'Nancy',
    postalCode: '54000',
    country: 'France',
  };
  const mockAddress2: WaypointDto = {
    lat: 48.8566,
    lon: 2.3522,
    locality: 'Paris',
    postalCode: '75000',
    country: 'France',
  };
  const mockAddress3: WaypointDto = {
    lon: 49.2628,
    lat: 4.0347,
    locality: 'Reims',
    postalCode: '51454',
    country: 'France',
  };

  it('should not validate if no position is defined', () => {
    expect(
      hasValidPositionIndexes([mockAddress1, mockAddress2, mockAddress3]),
    ).toBeFalsy();
  });
  it('should not validate if only one position is defined', () => {
    mockAddress1.position = 0;
    expect(
      hasValidPositionIndexes([mockAddress1, mockAddress2, mockAddress3]),
    ).toBeFalsy();
  });
  it('should not validate if positions are partially defined', () => {
    mockAddress1.position = 0;
    mockAddress2.position = null;
    mockAddress3.position = undefined;
    expect(
      hasValidPositionIndexes([mockAddress1, mockAddress2, mockAddress3]),
    ).toBeFalsy();
  });

  it('should not validate if multiple positions have same value', () => {
    mockAddress1.position = 0;
    mockAddress2.position = 1;
    mockAddress3.position = 1;
    expect(
      hasValidPositionIndexes([mockAddress1, mockAddress2, mockAddress3]),
    ).toBeFalsy();
  });
  it('should validate if all positions are ordered', () => {
    mockAddress1.position = 0;
    mockAddress2.position = 1;
    mockAddress3.position = 2;
    expect(
      hasValidPositionIndexes([mockAddress1, mockAddress2, mockAddress3]),
    ).toBeTruthy();
    mockAddress1.position = 1;
    mockAddress2.position = 2;
    mockAddress3.position = 3;
    expect(
      hasValidPositionIndexes([mockAddress1, mockAddress2, mockAddress3]),
    ).toBeTruthy();
  });
  it('should not validate if no waypoints are defined', () => {
    expect(hasValidPositionIndexes(undefined)).toBeFalsy();
    expect(hasValidPositionIndexes([])).toBeFalsy();
  });
});
